myname = 'find_LOCA_gridcells.R'

mpostscript( myname )

pid = Sys.getpid()
tmpfile = paste0( 'tmp.', myname, '.', pid )

source( 'utils.R' )

#======================================================================================
# Given:
#	target lon, lat : lon_t, lat_t
#	ibx, jbx : estimated closest point to target
#	xvals, yvals : arrays such that xvals[ibx] should be closest to lon_t, same for lat
#
check_pt = function( ipt, lon_t, lat_t, ibx, jbx, xvals, yvals ) {

	dist = array(0., dim=c(3,3))

	p1 = c( lon_t, lat_t )

	for( jj in -1:1 )
	for( ii in -1:1 ) {
		ixx = ibx + ii
		iyy = jbx + jj

		p2 = c( xvals[ixx], yvals[iyy] )

		dist[ii+2, jj+2] = great_circle_dist_haversine( p1, p2 )
		}

	dc = dist[2,2]
	mindist = min(dist)
	if( mindist != dc ) {
		print('===================')
		print( 'dist array:')
		print( dist )
		print(paste("Error, at pt", ipt, "found pt is not closest to target for lon, lat target=",
			lon_t, lat_t,
			'est indices:', ibx, jbx,
			'est lon, lat:', xvals[ibx], yvals[jbx],
			'which has dist', dc,
			'but actual least dist is', min(dist) ))

		for( jj in 1:3 )
		for( ii in 1:3 ) {
			if( mindist == dist[ii,jj] )
				return( c( ibx + ii - 2, jbx + jj - 2 ))
			}
		}

	return( c(ibx, jbx ))
}

#======================================================================================
calc_dat = function( opts ) {

	dat = list()

	pct = read_PCT()

	map( 'state' )

	lines( pct$lon, pct$lat, col='blue' )

	nn = length( pct$lon )
	xvals = pct$lat

	mplot( xvals, pct$elev )

	fname_templ = '/net/loca2/LOCA_2016-04-02/elevation.LOCA_2016-04-02.nc'

	rv = get_dat( fname_templ )

	xvals = rv$xvals
	yvals = rv$yvals

	dx = xvals[2] - xvals[1]
	dy = yvals[2] - yvals[1]

	ibx = array(0, nn)
	jbx = array(0, nn)

	for( ipt in 1:nn ) {
		ibx[ipt] = floor((pct$lon[ipt] - xvals[1]) / dx + 0.5) + 1
		jbx[ipt] = floor((pct$lat[ipt] - yvals[1]) / dy + 0.5) + 1
		}

	#-------------------------------
	# Check and make sure this works
	#-------------------------------
	for( ipt in 1:nn ) {
		if( ipt %% 100 == 0 )
			print(paste("Working on", ipt, '/', nn ))

		trv = check_pt( ipt, pct$lon[ipt], pct$lat[ipt], ibx[ipt], jbx[ipt], xvals, yvals )

		ibx[ipt] = trv[1]
		jbx[ipt] = trv[2]
		}

	ccontour( rv$xvals, rv$yvals, rv$dat, main='PCT on LOCA grid' )
	map( 'state', add=TRUE )

	#---------------------
	# Elim redundant boxes
	#---------------------
	ibx_last = -1
	jbx_last = -1
	nout = 0
	ibx2 = array(0, nn)
	jbx2 = array(0, nn)
	for( ipt in 1:nn ) {
		
		if( (ibx[ipt] != ibx_last) && (jbx[ipt] != jbx_last)) {
			nout = nout + 1
			ibx2[nout] = ibx[ipt]
			jbx2[nout] = jbx[ipt] 

			ibx_last = ibx[ipt]
			jbx_last = jbx[ipt]
			}
		}

	ibx = ibx2[1:nout]
	jbx = jbx2[1:nout]

	print(paste("before elim redundant boxes, n=", nn, "after=", nout ))

	xx = xvals[ibx]
	yy = yvals[jbx]
	lines( xx, yy, col='red' )

	fname_out = paste0( myname, '.points.txt' )
	fout = file( fname_out, 'w' )
	for( kk in 1:nout ) {
		cat( file=fout, ibx[kk], ' ', jbx[kk], ' ', xvals[ibx[kk]], ' ', yvals[jbx[kk]], '\n', sep='' )
		}

	close( fout )

	return(dat)
}

#======================================================================================
plotup = function( dat, opts, po ) {

}

#======================================================================================
doit = function() {

	tdate = date()
	print(paste("==================", myname, "starting at", tdate, '==============' ))

	use_cache = FALSE
	cache_fname = paste(myname,'_Rcache', sep='' )
	cache_file_exists = (filesize(cache_fname) > 10)

	#-----------------
	# Set data options
	#-----------------
	opts = list()

	#--------------------
	# Use cache, if asked
	#--------------------
	if( use_cache ) {
		if( cache_file_exists ) {
			print(paste("loading data from cache file", cache_fname))
			load( file=cache_fname )
			print('...done')
			do_calc_dat = FALSE
			}
		else
			do_calc_dat = TRUE
		}
	else
		do_calc_dat = TRUE

	#-------------------
	# Calculate the data
	#-------------------
	if( do_calc_dat )
		dat = calc_dat( opts )
		
	#-----------------------------------------------------
	# Save to cache if requested and doesn't exist already
	#-----------------------------------------------------
	if( use_cache && (! cache_file_exists))
		save( file=cache_fname, dat, opts )
			
	#---------------------------------
	# Set plot options and plot result
	#---------------------------------
	po = list()
	plotup( dat, opts, po )
}

#======================================================================================

doit()
mdev.off( myname )
