#================================================================================
get_data_slab = function( var, models, expt, year0, year1, include_daily=FALSE ) {

	nmods = length( models )

	for( imod in 1:nmods ) {
		fname = paste0( 'pct_data_cache/', models[imod], '.', var, '.', expt, '.PCT.nc' )

		print(paste("Reading", fname ))
		ncid = nc_open( fname )

		if( imod == 1 ) {
			tdim = ncid$dim[[ ncid$unlimdimid ]]
			dates = utCalendar( tdim$vals, tdim$units )

			keep = ((dates$year >= year0) & (dates$year <= year1))
			nkeep = sum( keep )
			if( nkeep < 100 ) stop(paste("Error nkeep=", nkeep, "for expt", expt, "years", year0, year1 ))

			nsta = ncid$dim$station$len

			dat_all = array(0., dim=c(nsta,nmods,nkeep))

			lon = ncvar_get( ncid, 'lon' )
			lat = ncvar_get( ncid, 'lat' )
			ix  = ncvar_get( ncid, 'ix' )
			iy  = ncvar_get( ncid, 'iy' )

			#------------------------------
			# apply year keep mask to dates
			#------------------------------
			dates = apply_keep_mask_to_dates( keep, dates )
			}

		tdat = ncvar_get( ncid, var )	# [nsta,nt]

		#--------------------------------------------------------
		# Some models end in 2099 rather than 2100 so adjust keep 
		# if that is true
		#--------------------------------------------------------
		nt_file = dim( tdat )[2]
		keep2use = keep[ 1:nt_file ]

		dat_all[,imod,] = tdat[,keep2use]

		nc_close( ncid )
		}

	#-----------------------------------------
	# Make climatology of slab for convenience
	#-----------------------------------------
	dat_climo = array(0., dim=c(nsta,nmods,12))
	for( imo in 1:12 ) {

		keep = (dates$month == imo)	# keep only this month

		for( imod in 1:nmods ) 
		for( ista in 1:nsta ) {

			tt = dat_all[ista,imod,keep]
			dat_climo[ista,imod,imo] = mean( tt, na.rm=TRUE )
			}
		}

	#--------------------------------
	# Get yearly mean for convenience
	#--------------------------------
	dat_annmean = array(0., dim=c(nsta,nmods))
	for( imod in 1:nmods ) 
	for( ista in 1:nsta ) {

		tt = dat_all[ista,imod,]
		dat_annmean[ista,imod] = mean( tt, na.rm=TRUE )
		}

	retval = list(
		dates	= dates,
		lon	= lon,
		lat	= lat,
		ix	= ix,
		iy	= iy,
		dat_annmean	= dat_annmean,	# [nsta,nmods]
		dat_climo	= dat_climo	# [nsta,nmods,12]
		)

	if( include_daily )
		retval$dat	= dat_all 	# [nsta,nmods,nt]

	return( retval )
}

#================================================================================
get_pct_pts = function() {

	t = reads( 'find_LOCA_gridcells.R.points.txt' )

	ix = t[,1]
	iy = t[,2]

	lon = t[,3]
	lat = t[,4]

	fname_elev = '/net/loca2/LOCA_2016-04-02/elevation.LOCA_2016-04-02.nc'
	rv = get_dat( fname_elev )
	np = length( ix )
	elev = array(0, np)

	for( ip in 1:np ) 
		elev[ip] = rv$dat[ix[ip], iy[ip] ]

	return( list(
		elev = elev,	# on the LOCA grid only
		ix = ix,
		iy = iy,
		lon = lon,
		lat = lat
		))
}

#==================================================================================
read_PCT = function() {

	t = dp_read_csv( '/home/pierce/projects/2021/Pacific_Crest_Trail_PCT/PCT_Centerline_Vertices.csv', nskip=1 )

	nn = length( t )

	lon = array(0., nn)
	lat = array(0., nn)
	elev = array(0., nn)

	for( kk in 1:nn ) {
		lat[kk] = as.double( t[[kk]][2] )
		lon[kk] = as.double( t[[kk]][3] )
		elev[kk] = as.double( t[[kk]][4] )
		}

	return( list(
		lon	= lon,
		lat	= lat,
		elev	= elev 
		))
}

#===============================================================================
label_PCT = function( dy=100) {

	t = reads( 'find_extrema.R.txt', is_char=TRUE )

	# 1 -116.4131 32.85839 1299.0 1 Mt._Laguna 

	lon = as.double( t[,2] )
	lat = as.double( t[,3] )
	elev = as.double( t[,4] )
	porv = as.integer( t[,5] )
	name_us = t[,6] 

	#------------------------
	# Possible values of porv
	#------------------------
	IVALL = 0
	IPEAK = 1

	nn = length( lon )

	name = array( ' ', nn )
	for( kk in 1:nn ) 
		name[kk] = gsub( '_', ' ', name_us[kk] )
	
	for( kk in 1:nn ) {

		if( porv[kk] == IVALL ) 	# valley
			y2use = elev[kk] - dy
		else			# peak
			y2use = elev[kk] + dy

		if( name_us[kk] == 'Mt._Baden-Powell' )
			yextra = 100
		else
			yextra = 0

		text( lat[kk], y2use+yextra, name[kk], adj=c(0.5, 0.5)) 

		xx = lat[kk]
		y0 = elev[kk]
		y1 = y2use + yextra
		}

}

